use v5.20;
use Test::More;

for my $madness
(
    qw
    (
        Object::Exercise::Execute
        Object::Exercise
    )
)
{
    use_ok $madness;

    ok $madness->can( 'VERSION' ),  "$madness can 'VERSION'.";
    ok $madness->VERSION,           "$madness has a version.";
}

done_testing;

__END__
